@@ -3,6 +3,8 @@ package main
 import (
 	"errors"
 	"fmt"
+
+	"github.com/sirupsen/logrus"
 	"github.com/yosuke-furukawa/json5/encoding/json5"
 )
 
@@ -44,6 +46,7 @@ type serverConfig struct {
 	ReceiveWindowClient uint64 `json:"recv_window_client"`
 	MaxConnClient       int    `json:"max_conn_client"`
 	DisableMTUDiscovery bool   `json:"disable_mtu_discovery"`
+	IPv6Only            bool   `json:"ipv6_only"`
 }
 
 func (c *serverConfig) Check() error {
@@ -70,6 +73,25 @@ func (c *serverConfig) String() string {
 	return fmt.Sprintf("%+v", *c)
 }
 
+type Relay struct {
+	Listen  string `json:"listen"`
+	Remote  string `json:"remote"`
+	Timeout int    `json:"timeout"`
+}
+
+func (r *Relay) Check() error {
+	if len(r.Listen) == 0 {
+		return errors.New("no relay listen address")
+	}
+	if len(r.Remote) == 0 {
+		return errors.New("no relay remote address")
+	}
+	if r.Timeout != 0 && r.Timeout <= 4 {
+		return errors.New("invalid relay timeout")
+	}
+	return nil
+}
+
 type clientConfig struct {
 	Server   string `json:"server"`
 	UpMbps   int    `json:"up_mbps"`
@@ -99,16 +121,10 @@ type clientConfig struct {
 		DNS     []string `json:"dns"`
 		Persist bool     `json:"persist"`
 	} `json:"tun"`
-	TCPRelay struct {
-		Listen  string `json:"listen"`
-		Remote  string `json:"remote"`
-		Timeout int    `json:"timeout"`
-	} `json:"relay_tcp"`
-	UDPRelay struct {
-		Listen  string `json:"listen"`
-		Remote  string `json:"remote"`
-		Timeout int    `json:"timeout"`
-	} `json:"relay_udp"`
+	TCPRelays []Relay `json:"relay_tcps"`
+	TCPRelay  Relay   `json:"relay_tcp"` // deprecated, but we still support it for backward compatibility
+	UDPRelays []Relay `json:"relay_udps"`
+	UDPRelay  Relay   `json:"relay_udp"` // deprecated, but we still support it for backward compatibility
 	TCPTProxy struct {
 		Listen  string `json:"listen"`
 		Timeout int    `json:"timeout"`
@@ -133,15 +149,10 @@ type clientConfig struct {
 func (c *clientConfig) Check() error {
 	if len(c.SOCKS5.Listen) == 0 && len(c.HTTP.Listen) == 0 && len(c.TUN.Name) == 0 &&
 		len(c.TCPRelay.Listen) == 0 && len(c.UDPRelay.Listen) == 0 &&
+		len(c.TCPRelays) == 0 && len(c.UDPRelays) == 0 &&
 		len(c.TCPTProxy.Listen) == 0 && len(c.UDPTProxy.Listen) == 0 {
 		return errors.New("please enable at least one mode")
 	}
-	if len(c.TCPRelay.Listen) > 0 && len(c.TCPRelay.Remote) == 0 {
-		return errors.New("no TCP relay remote address")
-	}
-	if len(c.UDPRelay.Listen) > 0 && len(c.UDPRelay.Remote) == 0 {
-		return errors.New("no UDP relay remote address")
-	}
 	if c.SOCKS5.Timeout != 0 && c.SOCKS5.Timeout <= 4 {
 		return errors.New("invalid SOCKS5 timeout")
 	}
@@ -151,12 +162,28 @@ func (c *clientConfig) Check() error {
 	if c.TUN.Timeout != 0 && c.TUN.Timeout < 4 {
 		return errors.New("invalid TUN timeout")
 	}
+	if len(c.TCPRelay.Listen) > 0 && len(c.TCPRelay.Remote) == 0 {
+		return errors.New("no TCP relay remote address")
+	}
+	if len(c.UDPRelay.Listen) > 0 && len(c.UDPRelay.Remote) == 0 {
+		return errors.New("no UDP relay remote address")
+	}
 	if c.TCPRelay.Timeout != 0 && c.TCPRelay.Timeout <= 4 {
 		return errors.New("invalid TCP relay timeout")
 	}
 	if c.UDPRelay.Timeout != 0 && c.UDPRelay.Timeout <= 4 {
 		return errors.New("invalid UDP relay timeout")
 	}
+	for _, r := range c.TCPRelays {
+		if err := r.Check(); err != nil {
+			return err
+		}
+	}
+	for _, r := range c.UDPRelays {
+		if err := r.Check(); err != nil {
+			return err
+		}
+	}
 	if c.TCPTProxy.Timeout != 0 && c.TCPTProxy.Timeout <= 4 {
 		return errors.New("invalid TCP TProxy timeout")
 	}
@@ -173,6 +200,12 @@ func (c *clientConfig) Check() error {
 		(c.ReceiveWindow != 0 && c.ReceiveWindow < 65536) {
 		return errors.New("invalid receive window size")
 	}
+	if len(c.TCPRelay.Listen) > 0 {
+		logrus.Warn("'relay_tcp' is deprecated, please use 'relay_tcps' instead")
+	}
+	if len(c.UDPRelay.Listen) > 0 {
+		logrus.Warn("config 'relay_udp' is deprecated, please use 'relay_udps' instead")
+	}
 	return nil
 }
 
